/*   changes from v7
- count and compare the two clocks and the two BC0's
changes from v6  
- spy and check HTR data at 80 MHz
- added a 2nd DCM for TTC_CK
changes from v5  
- Add a bit: "Disable Spy On Valid Data" DisSODV on CntrReg
changes from v4:
- Increased depth of Vitesse Spy Fifos
changes from v3:
- Added VitResetN on WriteOnlyCntrReg
- use both clocks
- TTC for the HTR inputs, RxClk for the Vitesse
- spy HTR data with TTC clock.
changes from v2:
- swapped input clocks of BUFGMUX
- added register for real time value of Vitesse flags.
- change addresses to make some commands compatible with SLB:
  START <=> write a 2 on addr 0x61    ---> Start SpyFIFOs
  STOP  <=> write a 4 on addr 0x61    ---> Stop  SpyFIFOs
  STATUS<=> read addr 0x60, 0x70      ---> read Vitesse_StatReg and Fifo_StatReg
  CSTAT <=> read from 0x0 or 0x10     ---> read CntrReg or StatusReg
  REGS  <=> read 0x60 and either 0x62-6F or 0x72-7F  ---> unused in SLBS

TBD 
*/
/* Note that all 120 Mb/s inputs from the Vitesse are synchronous with
REFCLK. The inputs are valid and stable on rising edges of REFCLK 
plus a round-trip propagation delay (? ns) */

`include "define.v"
module slbsTop 
 ( 
 input  CLKPLUS, CLKMINUS, // 40MHz differential RX_CLK
 input  BCOPLUS, BCOMINUS, // RX_BC0 common to all SLB boards
 input  TTC_CK40Des1, TTC_BRCST_STRB1, TTC_BCR,
 input  LOC_C_S,  LOC_R_W, 
 input  ErrA, ErrB, ErrC, ErrD,        //  120 Mb/s inputs
        IdleA, IdleB, IdleC, IdleD,    //  120 Mb/s inputs
        KChA, KChB, KChC, KChD,        //  120 Mb/s inputs
 input  [7:0] RecA, RecB, RecC, RecD,  //  120 Mb/s inputs
 input  [7:2] TTC_BRCST,      
 input  [8:0] SYNC1, SYNC2, SYNC3, SYNC4, 
              SYNC5, SYNC6, SYNC7, SYNC8,   // Data In
 input  [`LB_Addr_Size-1:0]  LOCADD,  // The Address bus.
 inout  [`REG_WIDTH-1:0]     LOCBUS,  // The bidir Data bus 

 output TPMINUS, TPPLUS, // to RJ45 J8
 output TRSTN, 
        REFCLK,     // to MC100EPT22 and then to Vitesse mezzanine
        PECL_OUT,  // to two MC100EPT22 and then connectors
        VitResetN          );

 wire   
  TTC_Hard_rst, TTC_Soft_rst, TTC_Start, TTC_Stop, 
  VME_Hard_rst, VME_Soft_rst, VME_Start, VME_Stop, VME_LVDSout,
  run_status, DisSODV, BunchEr, GateTTCcommands, Hard_rst, Soft_rst;
 wire [7:0] LastTTCcommand;
 wire [15:0] TTC_test0, TTC_test1, TTC_test2, TTC_test3,
             TTC_test4, TTC_test5, TTC_test6, TTC_test7,
             TTC_test8, TTC_test9, TTC_testA, TTC_testB,
             TTC_testC, TTC_testD, TTC_testE, TTC_testF;
 wire [`BCN_WIDTH-1:0] MaxBCN; 
 wire [8:1]  TPSpyFifoEmpty, TPSpyFifoFull, TPSpyFifoRd,
             InstantCntErr,  InstantR1sErr;
 wire [15:0] AccCntErr, AccR1sErr;  
 wire [4:1]  VitSpyFifoEmpty, VitSpyFifoFull, VitSpyFifoRd;     
 wire [8:0]  TPSpyFIFO1_out, TPSpyFIFO2_out, TPSpyFIFO3_out, TPSpyFIFO4_out,
             TPSpyFIFO5_out, TPSpyFIFO6_out, TPSpyFIFO7_out, TPSpyFIFO8_out;
 wire [10:0] VitSpyFIFO1_out, VitSpyFIFO2_out,
             VitSpyFIFO3_out, VitSpyFIFO4_out;	
 wire [6:0]  pad7_1,  pad7_2,  pad7_3,  pad7_4;
 wire [26:0] pad27_1, pad27_2, pad27_3, pad27_4,
             pad27_5, pad27_6, pad27_7, pad27_8;
 reg SpyVit;
 reg ErrA_2D, ErrA_D, IdleA_2D, IdleA_D, KChA_2D, KChA_D, 
     ErrB_2D, ErrB_D, IdleB_2D, IdleB_D, KChB_2D, KChB_D, 
     ErrC_2D, ErrC_D, IdleC_2D, IdleC_D, KChC_2D, KChC_D,
     ErrD_2D, ErrD_D, IdleD_2D, IdleD_D, KChD_2D, KChD_D;  
 reg [7:0] RecA_2D, RecA_D, RecB_2D, RecB_D, RecC_2D, RecC_D, RecD_2D, RecD_D;  
 reg [8:0] SYNC1D, SYNC2D, SYNC3D, SYNC4D, SYNC5D, SYNC6D, SYNC7D, SYNC8D;
 reg [15:0] RxClkCnt, RxBC0Cnt, TClkCnt, TxBC0Cnt;


//  ********************************************************************
//      ********************  CLOCK SECTION    *****************
//            *********************************************

 wire   RxClk_Buf, RxClk, Rclk2x, clk0, clk2x_dll;
 wire   DLL_lock, ClkFx, clk3x;
 wire   [7:0] DLL_status, TTC_DLL_status; // status[1] asserted if .CLKIN stopped
 wire   TTC_CK_Buf, Tclk0, TTC_CK, Tclk2x_dll, TTC_CK2x, TTC_DLL_lock;
 reg    ResetDLL, DLL_lock_D;
 reg    [14:0] RstCnt; // counter for proper reset of DLL 
 reg    [2:0] UnlockCnt; // counter for proper reset of DLL 

 // Instantiate the Differential Global Input Buffer 
  IBUFGDS_LVPECL_25 difclkbuf (.I(CLKPLUS), .IB(CLKMINUS), .O(RxClk_Buf));
  IBUFG ttcclkbuf ( .I(TTC_CK40Des1), .O(TTC_CK_Buf) );

// Generate a DLL reset (3-ticks) after FPGA is configured, see:
// www.xilinx.com/publications/xcellonline/xcell_51/xc_particle51.htm
  FDS flop1 (.D(1'b0), .C(RxClk_Buf), .Q(out1), .S(1'b0)); 
  FD  flop2 (.D(out1), .C(RxClk_Buf), .Q(out2)); 
  FD  flop3 (.D(out2), .C(RxClk_Buf), .Q(out3)); 
  FD  flop4 (.D(out3), .C(RxClk_Buf), .Q(out4)); 
  assign ConfigRst = (out2 | out3 | out4); 

  DCM TTC_DCM
   (.CLKIN(TTC_CK_Buf), .CLKFB(TTC_CK),  .RST(ConfigRst),  
    .CLK0(Tclk0),   .CLK90(),  .CLK180(),  .CLK270(),
    .CLK2X(),  .CLK2X180(Tclk2x_dll),  // here u can use the inverted clock.
    .CLKDV(),   .CLKFX(),   .CLKFX180(),
    .LOCKED(TTC_DLL_lock),  .STATUS(TTC_DLL_status),
    .DSSEN(1'b0),
    .PSCLK(1'b0),  .PSEN(1'b0),  .PSINCDEC(1'b0), .PSDONE() );

  BUFG TCK40BUF (.I(Tclk0), .O(TTC_CK));  
  BUFG TCK2XBUF (.I(Tclk2x_dll), .O(TTC_CK2x)); 

  DCM RxClk_DCM
   (.CLKIN(RxClk_Buf), .CLKFB(RxClk),  .RST(ConfigRst),  
    .CLK0(clk0),   .CLK90(),  .CLK180(),  .CLK270(),
    .CLK2X(clk2x_dll),  .CLK2X180(),
    .CLKDV(),   .CLKFX(ClkFx),   .CLKFX180(),
    .LOCKED(DLL_lock),  .STATUS(DLL_status),
    .DSSEN(1'b0),
    .PSCLK(1'b0),  .PSEN(1'b0),  .PSINCDEC(1'b0), .PSDONE() );

  //synthesis attribute DFS_FREQUENCY_MODE of MainDCM is "LOW" 
  //synthesis attribute CLKFX_DIVIDE of MainDCM is "1" 
  //synthesis attribute CLKFX_MULTIPLY of MainDCM is "3" 
  //synthesis attribute STARTUP_WAIT of MainDCM is "FALSE" 

/* If tests show that DCM looses lock, consider to
   increase the FACTORY_JF setting to 'FFFF  (Xilinx Answer 13756).
   Anyway lab test in Dec03 showed that the DLL relock spontaneously
   when the TTCrx was loosing lock and then relocking   */
/* If there is a chance that RxClk_Buf stops completly, the LOCKED 
   signal will not update; consider the STATUS bit "CLK_IN_STOPPED".
   I do not implement it as I don't expect this occurence, although
   trivial mechanical troubles of the TTC-PMC mezzanine could do it. 
   Note that if the TTCrx looses lock, TTC_clk will keep toggling at 
   the TTCrx natural oscillation frequency, but will not stop.    */

// ResetDLL has some "feedback" problems when connected to DCM,
// so far I don't use it.
 always @ (posedge TTC_CK) begin  //TTC_CK should toggle even if no cable 
    if (DLL_lock == 0) begin
         RstCnt <= RstCnt + 1;
         if (RstCnt == 15'h 7FFF)
               ResetDLL <= 1;  // reset no more then every 2^15 25ns
         else     ResetDLL <= 0;
    end
    DLL_lock_D <= DLL_lock;
    if (Hard_rst)  UnlockCnt <= 0;
    else if (DLL_lock_D & (~DLL_lock) & (UnlockCnt != 3'b111)) //hi->lo transition
             UnlockCnt <= UnlockCnt + 1;   // increment (stop at max count)
 end //always 

 BUFG CK40BUF (.I(clk0), .O(RxClk));  // Use RxClk as internal clock 
 BUFG CK2XBUF (.I(clk2x_dll), .O(Rclk2x)); 
 BUFG CK3XBUF    (.I(ClkFx),    .O(clk3x));

 assign REFCLK = clk3x; // to U2A and then to Wisc_Mezzanine.

//       *************  END OF CLOCK SECTION  *****************
//  *****************************************************************

 IBUFDS_LVPECL_25 RxBC0buf (.I(BCOPLUS), .IB(BCOMINUS), .O(RxBC0));
 assign PECL_OUT = RxBC0;
 assign TRSTN = 1'b1; // Wisc. Rec_Mezz JTAG reset.
 OBUFDS_LVPECL_25 RJ45pair (.I(VME_LVDSout), .O(TPPLUS), .OB(TPMINUS));


 wire DataModeA = ~(ErrA | IdleA | KChA);
 always @ (posedge clk3x) begin
   SpyVit <= run_status & (DataModeA | DisSODV);
   ErrA_D <= ErrA;   IdleA_D <= IdleA;  KChA_D <= KChA;  RecA_D <= RecA;  
   ErrA_2D<= ErrA_D; IdleA_2D<=IdleA_D; KChA_2D<=KChA_D; RecA_2D<=RecA_D;  
   ErrB_D <= ErrB;   IdleB_D <= IdleB;  KChB_D <= KChB;  RecB_D <= RecB;  
   ErrB_2D<= ErrB_D; IdleB_2D<=IdleB_D; KChB_2D<=KChB_D; RecB_2D<=RecB_D;  
   ErrC_D <= ErrC;   IdleC_D <= IdleC;  KChC_D <= KChC;  RecC_D <= RecC;  
   ErrC_2D<= ErrC_D; IdleC_2D<=IdleC_D; KChC_2D<=KChC_D; RecC_2D<=RecC_D;  
   ErrD_D <= ErrD;   IdleD_D <= IdleD;  KChD_D <= KChD;  RecD_D <= RecD;  
   ErrD_2D<= ErrD_D; IdleD_2D<=IdleD_D; KChD_2D<=KChD_D; RecD_2D<=RecD_D;  
 end


// **** 4 Vitesse Spy FIFOs (1 per Vitesse channel) ***
// 2 BRAM per channel : 2047-deep fifos

FifoIc2047x18  VitSpyFIFO1 
 ( .fifo_gsr_in(Hard_rst),          .write_data_in({7'b0, ErrA_2D, IdleA_2D, KChA_2D, RecA_2D}), 
   .write_clock_in(clk3x),          .write_enable_in(SpyVit),
   .read_clock_in(RxClk),           .read_enable_in(VitSpyFifoRd[1]),
   .read_data_out({pad7_1, VitSpyFIFO1_out}),  .fifostatus_out(),
   .full_out(VitSpyFifoFull[1]),    .empty_out(VitSpyFifoEmpty[1])  );

FifoIc2047x18  VitSpyFIFO2 
 ( .fifo_gsr_in(Hard_rst),          .write_data_in({7'b0, ErrB_2D, IdleB_2D, KChB_2D, RecB_2D}), 
   .write_clock_in(clk3x),          .write_enable_in(SpyVit),
   .read_clock_in(RxClk),           .read_enable_in(VitSpyFifoRd[2]),
   .read_data_out({pad7_2, VitSpyFIFO2_out}),  .fifostatus_out(),
   .full_out(VitSpyFifoFull[2]),    .empty_out(VitSpyFifoEmpty[2])  );

FifoIc2047x18  VitSpyFIFO3 
 ( .fifo_gsr_in(Hard_rst),          .write_data_in({7'b0, ErrC_2D, IdleC_2D, KChC_2D, RecC_2D}), 
   .write_clock_in(clk3x),          .write_enable_in(SpyVit),
   .read_clock_in(RxClk),           .read_enable_in(VitSpyFifoRd[3]),
   .read_data_out({pad7_3, VitSpyFIFO3_out}),  .fifostatus_out(),
   .full_out(VitSpyFifoFull[3]),    .empty_out(VitSpyFifoEmpty[3])  );

FifoIc2047x18  VitSpyFIFO4 
 ( .fifo_gsr_in(Hard_rst),          .write_data_in({7'b0, ErrD_2D, IdleD_2D, KChD_2D, RecD_2D}), 
   .write_clock_in(clk3x),          .write_enable_in(SpyVit),
   .read_clock_in(RxClk),           .read_enable_in(VitSpyFifoRd[4]),
   .read_data_out({pad7_4, VitSpyFIFO4_out}),  .fifostatus_out(),
   .full_out(VitSpyFifoFull[4]),    .empty_out(VitSpyFifoEmpty[4])  );

// **** Register, Spy and Check the HTR data ***

 always @ (posedge TTC_CK2x) begin  // register data as they go to 2 destinations.
  SYNC1D <= SYNC1;  SYNC2D <= SYNC2;  SYNC3D <= SYNC3;  SYNC4D <= SYNC4;  
  SYNC5D <= SYNC5;  SYNC6D <= SYNC6;  SYNC7D <= SYNC7;  SYNC8D <= SYNC8;  
 end

fifo_ic_v2  TPSpyFIFO1 
 ( .fifo_gsr_in(Hard_rst),            .write_data_in({27'b0, SYNC1D}), 
   .write_clock_in(TTC_CK2x),         .write_enable_in(run_status),
   .read_clock_in(RxClk),               .read_enable_in(TPSpyFifoRd[1]),
   .read_data_out({pad27_1, TPSpyFIFO1_out}), .fifostatus_out(),
   .full_out(TPSpyFifoFull[1]),       .empty_out(TPSpyFifoEmpty[1])  );

fifo_ic_v2  TPSpyFIFO2 
 ( .fifo_gsr_in(Hard_rst),            .write_data_in({27'b0, SYNC2D}), 
   .write_clock_in(TTC_CK2x),         .write_enable_in(run_status),
   .read_clock_in(RxClk),               .read_enable_in(TPSpyFifoRd[2]),
   .read_data_out({pad27_2, TPSpyFIFO2_out}), .fifostatus_out(),
   .full_out(TPSpyFifoFull[2]),       .empty_out(TPSpyFifoEmpty[2])  );

fifo_ic_v2  TPSpyFIFO3 
 ( .fifo_gsr_in(Hard_rst),            .write_data_in({27'b0, SYNC3D}), 
   .write_clock_in(TTC_CK2x),         .write_enable_in(run_status),
   .read_clock_in(RxClk),               .read_enable_in(TPSpyFifoRd[3]),
   .read_data_out({pad27_3, TPSpyFIFO3_out}), .fifostatus_out(),
   .full_out(TPSpyFifoFull[3]),       .empty_out(TPSpyFifoEmpty[3])  );

fifo_ic_v2  TPSpyFIFO4 
 ( .fifo_gsr_in(Hard_rst),            .write_data_in({27'b0, SYNC4D}), 
   .write_clock_in(TTC_CK2x),         .write_enable_in(run_status),
   .read_clock_in(RxClk),               .read_enable_in(TPSpyFifoRd[4]),
   .read_data_out({pad27_4, TPSpyFIFO4_out}), .fifostatus_out(),
   .full_out(TPSpyFifoFull[4]),       .empty_out(TPSpyFifoEmpty[4])  );

fifo_ic_v2  TPSpyFIFO5 
 ( .fifo_gsr_in(Hard_rst),            .write_data_in({27'b0, SYNC5D}), 
   .write_clock_in(TTC_CK2x),         .write_enable_in(run_status),
   .read_clock_in(RxClk),               .read_enable_in(TPSpyFifoRd[5]),
   .read_data_out({pad27_5, TPSpyFIFO5_out}), .fifostatus_out(),
   .full_out(TPSpyFifoFull[5]),       .empty_out(TPSpyFifoEmpty[5])  );

fifo_ic_v2  TPSpyFIFO6 
 ( .fifo_gsr_in(Hard_rst),            .write_data_in({27'b0, SYNC6D}), 
   .write_clock_in(TTC_CK2x),         .write_enable_in(run_status),
   .read_clock_in(RxClk),               .read_enable_in(TPSpyFifoRd[6]),
   .read_data_out({pad27_6, TPSpyFIFO6_out}), .fifostatus_out(),
   .full_out(TPSpyFifoFull[6]),       .empty_out(TPSpyFifoEmpty[6])  );

fifo_ic_v2  TPSpyFIFO7 
 ( .fifo_gsr_in(Hard_rst),            .write_data_in({27'b0, SYNC7D}), 
   .write_clock_in(TTC_CK2x),         .write_enable_in(run_status),
   .read_clock_in(RxClk),               .read_enable_in(TPSpyFifoRd[7]),
   .read_data_out({pad27_7, TPSpyFIFO7_out}), .fifostatus_out(),
   .full_out(TPSpyFifoFull[7]),       .empty_out(TPSpyFifoEmpty[7])  );

fifo_ic_v2  TPSpyFIFO8 
 ( .fifo_gsr_in(Hard_rst),            .write_data_in({27'b0, SYNC8D}), 
   .write_clock_in(TTC_CK2x),         .write_enable_in(run_status),
   .read_clock_in(RxClk),               .read_enable_in(TPSpyFifoRd[8]),
   .read_data_out({pad27_8, TPSpyFIFO8_out}), .fifostatus_out(),
   .full_out(TPSpyFifoFull[8]),       .empty_out(TPSpyFifoEmpty[8])  );

ErrorChecker ErrorChecker (
    .TTC_CK2x(TTC_CK2x),   .reset(Soft_rst), 
    .SYNC1(SYNC1D),        .SYNC2(SYNC2D), 
    .SYNC3(SYNC3D),        .SYNC4(SYNC4D), 
    .SYNC5(SYNC5D),        .SYNC6(SYNC6D), 
    .SYNC7(SYNC7D),        .SYNC8(SYNC8D), 
    .InstantCntErr(InstantCntErr),  .InstantR1sErr(InstantR1sErr), 
    .AccCntErr(AccCntErr),          .AccR1sErr(AccR1sErr)    );

////////////////////////////////////////////////////////

LocBusSlave LocBusSlave (
    .clk(RxClk),           .rst(Hard_rst), 
    .LocAddr(LOCADD),    .LocData(LOCBUS), 
    .LocCS_b(LOC_C_S),   .LocWRITE_b(LOC_R_W), 
    .LB_STATE(),    .Din(),  .Dout(),      .Tri(), 
    .DLL_lock(DLL_lock),    .UnlockCnt(UnlockCnt),
    .ErrA(ErrA), .IdleA(IdleA), .KChA(KChA), 
    .ErrB(ErrB), .IdleB(IdleB), .KChB(KChB),
    .ErrC(ErrC), .IdleC(IdleC), .KChC(KChC),
    .ErrD(ErrD), .IdleD(IdleD), .KChD(KChD),
    .LastTTCcommand(LastTTCcommand), .GateTTCcommands(GateTTCcommands), 
    .VME_Hard_rst(VME_Hard_rst),     .VME_Soft_rst(VME_Soft_rst), 
    .VME_Start(VME_Start),           .VME_Stop(VME_Stop), 
    .VME_VitResetN(VitResetN),       .VME_LVDSout(VME_LVDSout), 
    .run_status(run_status),         .DisSODV(DisSODV),
    .InstantCntErr(InstantCntErr),   .InstantR1sErr(InstantR1sErr), 
    .AccCntErr(AccCntErr),           .AccR1sErr(AccR1sErr),
    .RxClkCnt(RxClkCnt), .RxBC0Cnt(RxBC0Cnt),
    .TClkCnt(TClkCnt), .TxBC0Cnt(TxBC0Cnt),
    .TPSpyFifoRd(TPSpyFifoRd),
    .TPSpyFifoEmpty(TPSpyFifoEmpty),   .TPSpyFifoFull(TPSpyFifoFull), 
    .TPSpyFIFO1_out(TPSpyFIFO1_out),   .TPSpyFIFO2_out(TPSpyFIFO2_out),
    .TPSpyFIFO3_out(TPSpyFIFO3_out),   .TPSpyFIFO4_out(TPSpyFIFO4_out),
    .TPSpyFIFO5_out(TPSpyFIFO5_out),   .TPSpyFIFO6_out(TPSpyFIFO6_out),
    .TPSpyFIFO7_out(TPSpyFIFO7_out),   .TPSpyFIFO8_out(TPSpyFIFO8_out),
    .VitSpyFifoEmpty(VitSpyFifoEmpty), .VitSpyFifoFull(VitSpyFifoFull),
    .VitSpyFifoRd(VitSpyFifoRd),      
    .VitSpyFIFO1_out(VitSpyFIFO1_out), .VitSpyFIFO2_out(VitSpyFIFO2_out),
    .VitSpyFIFO3_out(VitSpyFIFO3_out), .VitSpyFIFO4_out(VitSpyFIFO4_out),
    .TTC_test0(TTC_test0),       .TTC_test1(TTC_test1), 
    .TTC_test2(TTC_test2),       .TTC_test3(TTC_test3), 
    .TTC_test4(TTC_test4),       .TTC_test5(TTC_test5), 
    .TTC_test6(TTC_test6),       .TTC_test7(TTC_test7), 
    .TTC_test8(TTC_test8),       .TTC_test9(TTC_test9), 
    .TTC_testA(TTC_testA),       .TTC_testB(TTC_testB), 
    .TTC_testC(TTC_testC),       .TTC_testD(TTC_testD), 
    .TTC_testE(TTC_testE),       .TTC_testF(TTC_testF), 
    .MaxBCN(MaxBCN),             .BunchEr(BunchEr), 
    .CounterModeSel()    );

  TTC_BrcstDecoder TTC_BrcstDecoder (
   .clk(RxClk),      .clk2x(Rclk2x),       .reset(VME_Hard_rst),
   .tTTC_BCntRes(TTC_BCR),              .TTC_Brcst(TTC_BRCST),
   .TTC_BrcstStr(TTC_BRCST_STRB1),      .TTC_CK40Des1(TTC_CK),  
   .LastTTCcommand(LastTTCcommand),     .TTC_Resync(),
   .TTC_Hard_rst(TTC_Hard_rst),         .TTC_Soft_rst(TTC_Soft_rst),  
   .TTC_Start(TTC_Start),               .TTC_Stop(TTC_Stop),
   .TTC_BC0(TTC_BC0),                  
   .TTC_AsyncFifoRst(), .TTC_TestEnable(TTC_TestEnable),
   .TTC_test0(TTC_test0), .TTC_test1(TTC_test1), .TTC_test2(TTC_test2), 
   .TTC_test3(TTC_test3), .TTC_test4(TTC_test4), .TTC_test5(TTC_test5),
   .TTC_test6(TTC_test6), .TTC_test7(TTC_test7), .TTC_test8(TTC_test8),
   .TTC_test9(TTC_test9), .TTC_testA(TTC_testA), .TTC_testB(TTC_testB),
   .TTC_testC(TTC_testC), .TTC_testD(TTC_testD), .TTC_testE(TTC_testE),
   .TTC_testF(TTC_testF),   .MaxBCN(MaxBCN),   .BunchEr(BunchEr),
   .BCN(),   .OrN() );

 commander commander (  
  .clk(RxClk),    .GateTTCcommands(GateTTCcommands),
  .TTC_Hard_rst(TTC_Hard_rst), .TTC_Soft_rst(TTC_Soft_rst),
  .TTC_Start(TTC_Start), .TTC_Stop(TTC_Stop), 
  .VME_Hard_rst(VME_Hard_rst), .VME_Soft_rst(VME_Soft_rst), 
  .VME_Start(VME_Start), .VME_Stop(VME_Stop), 
  .Hard_rst(Hard_rst),  .Soft_rst(Soft_rst), 
  .run_status(run_status)       );

 always @ (posedge RxClk) begin 
   if (Soft_rst) begin
      RxClkCnt <= 0;  RxBC0Cnt <= 0; TxBC0Cnt <= 0; 
   end
   else if (run_status) begin
      RxClkCnt <= RxClkCnt + 1;
      if (RxBC0) RxBC0Cnt <= RxBC0Cnt + 1;
      if (TTC_BC0) TxBC0Cnt <= TxBC0Cnt + 1;
   end
 end // always

 always @ (posedge TTC_CK) begin 
   if (Soft_rst)  TClkCnt <= 0; 
   else if (run_status) TClkCnt <= TClkCnt + 1;
 end // always

endmodule